/* SimpleStorage.cpp -- realization SimpleStorage extension of Onion registry
			library

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

#include "simplestorage.h"

using namespace Onion;

void SimpleStorage::open(bool read)
{
	if(read)
	{
		if(ofile != 0) 
			throw StorageAlreadyOpen( "for writing" );
		if(ifile != 0)
			throw StorageAlreadyOpen( "for reading" );
			
		ifile = new std::ifstream(name.c_str());
		if(ifile == 0)
		{
			state = CLOSED;
			throw CantCreateStorage( "can't create ifstream");
		}
		if(!ifile->is_open())
		{
			delete ifile;
			state = CLOSED;
			throw CantOpenStorage( "can't open ifstream" );
		}
		state = LOAD;
	}
	else
	{
		if(ofile != 0) 
			throw StorageAlreadyOpen( "for writing" );
		if(ifile != 0)
			throw StorageAlreadyOpen( "for reading" );
			
		ofile = new std::ofstream(name.c_str());
		if(ofile == 0)
		{
			state = CLOSED;
			throw CantCreateStorage( "can't create ofstream" );
		}
		if(!ofile->is_open())
		{
			delete ofile;
			state = CLOSED;
			throw CantOpenStorage( "can't open ofstream" );
		}
		state = STORE;
	}
}

void SimpleStorage::close()
{
	if(ifile)
	{
		delete ifile;
		ifile = 0;
	}
	if(ofile)
	{
		delete ofile;
		ofile = 0;
	}
	state = CLOSED;
}

void SimpleStorage::save(const std::string &variable, const std::string &value)
{
	if(state != STORE)
		throw ErrorException("open(false) Storage before saving");

	*ofile << variable << std::endl << value << std::endl;
}

void SimpleStorage::saveMarker(const std::string &marker,
	const std::string &value)
{
	if(state != STORE)
		throw ErrorException("open Storage before saveMarker\n");
		
	*ofile << std::endl << marker << std::endl << value << std::endl;
}

bool SimpleStorage::load(std::string &variable, std::string &value)
{
	if(state != LOAD)
		throw ErrorException( "open storage before loading\n" );

	if(ifile->fail())
		return false;

	char buffer[1025];
	ifile->getline(buffer, 1024);

	std::string line(buffer);

	if(line == "") // marker
	{
		ifile->getline(buffer, 1024);
		variable = buffer;

		ifile->getline(buffer, 1024);
		value = buffer;
	
		return false;
	}

	variable = buffer;

	ifile->getline(buffer, 1024);
	value = buffer;

	return true;
}
