/* registry.cpp -- realization of Onion Registry library

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */


//#include <iostream>

#include "registry.h"

using namespace Onion;

Key::Key() throw(std::bad_alloc)
{
}

Key::Key(const std::string &val) throw(std::bad_alloc) : value(val)
{
}

Key &Key::operator=(const std::string &val) throw()
{
	value = val;
}

std::string Key::getValue() const throw(std::bad_alloc)
{
	return value;
}

Key::operator std::string() const throw(std::bad_alloc)
{
	return value;
}

void Key::save( bool recurse ) const
{
	Storage *storage = Storage::makeStorage( *this );
	storage->open( false );
	save( *storage, recurse );
	storage->close();
	delete storage;
}

void Key::load( bool recurse )
{
	Storage *storage = Storage::makeStorage( *this );
	storage->open( true );
	load( *storage, recurse );
	storage->close();
	delete storage;
}

void Key::save(Storage &storage, bool recurse ) const
{
	storage.saveMarker( "begin", "key" );
	Node::save( storage, recurse );
	storage.save( "value", value );
	storage.saveMarker( "end", "key" );
}

void Key::load(Storage &storage, bool recurse )
{
	std::string var, val;
	if( ( storage.load( var, val ) != false )
		|| ( var != "begin" ) || ( val != "key" ) )
		throw MarkerExpected( "begin::key" );
	Node::load( storage, recurse );
	if( storage.load( var, val ) != true )
		throw VariableExpected( "any variable" );

	if( var != "value" )
		throw BadVariable( var );

	value = val;

	if( ( storage.load( var, val ) != false )
		|| ( var != "end" ) || (val != "key" ) )
		throw MarkerExpected( "end::key" );
}

// <EOF>
