#ifndef TESTS_KEYTEST_H
#define TESTS_KEYTEST_H

#include "cppunit/extensions/HelperMacros.h"
#include "../src/registry.h"
#include <iostream>
#include <algorithm>

class KeyTest : public CppUnit::TestFixture
{
	CPPUNIT_TEST_SUITE( KeyTest );

	CPPUNIT_TEST( testGetValue );
	CPPUNIT_TEST( testOperatorString );
	CPPUNIT_TEST( testOperatorAssign );
	
	CPPUNIT_TEST( testSave );
	CPPUNIT_TEST( testLoad );

	CPPUNIT_TEST_SUITE_END();	

protected:
	Onion::Key *m_key;
	Onion::Registry *m_registry;
public:
	int countTestCases() const
	{
		return 1;
	}
	
	void setUp()
	{
		m_registry = new Onion::Registry();
		Onion::Catalog *root = m_registry->getRoot();
		Onion::Catalog *catalog = root->createCatalog( "catalog" );
		m_key = catalog->createKey( "key", "value" );
	}
	
	void tearDown()
	{
		delete m_registry;
	}

	void testGetValue()
	{
		CPPUNIT_ASSERT( m_registry != 0 );
		CPPUNIT_ASSERT( m_key != 0 );
		CPPUNIT_ASSERT( m_key->getValue() == "value");
	}

	void testOperatorString()
	{
		std::string value = *m_key;
		CPPUNIT_ASSERT( value == "value" );
	}

	
	void testOperatorAssign()
	{
		*m_key = "value2";
		CPPUNIT_ASSERT( m_key->getValue() == "value2" );
	}
	
	void testSave()
	{
		Onion::MetaKey *mkey;
		mkey = m_key->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_key->createMetaKey( "filename", "testKey.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		*m_key = "saved value";

		m_key->save( false );
	}
	
	void testLoad()
	{
		Onion::MetaKey *mkey;
		mkey = m_key->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_key->createMetaKey( "filename", "testKey.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		CPPUNIT_ASSERT( m_key->getValue() == "value" );
		m_key->load( false );
		CPPUNIT_ASSERT( m_key->getValue() == "saved value" );
	}
};

#endif // TESTS_KEYTEST_H

// <EOF>
