#ifndef TESTS_NODETEST_H
#define TESTS_NODETEST_H

#include "cppunit/extensions/HelperMacros.h"
#include "../src/registry.h"
//#include <iostream>
#include <algorithm>

#include <stdlib.h>

class NodeTest : public CppUnit::TestFixture
{
	CPPUNIT_TEST_SUITE( NodeTest );

	CPPUNIT_TEST( testExistance );
	CPPUNIT_TEST( testCreateMetaKey );
	CPPUNIT_TEST( testGetMetaKey );
	CPPUNIT_TEST_EXCEPTION( testMetaKeyNotFound, Onion::MetaKeyNotFound );

	CPPUNIT_TEST( testGetMetaKeyList );
	CPPUNIT_TEST( testRemoveMetaKey );

	CPPUNIT_TEST( testSave );
	CPPUNIT_TEST( testLoad );

	CPPUNIT_TEST_SUITE_END();
protected:
	Onion::Registry *m_registry;
	Onion::Node *m_node;
public:

	int countTestCases() const
	{
		return 1;
	}

	void setUp()
	{
		m_registry = new Onion::Registry;
		Onion::Catalog *root = m_registry->getRoot();
		m_node =  root->createCatalog("catalog");
	}

	void tearDown()
	{
		delete m_registry;
	}

	void testExistance()
	{
		CPPUNIT_ASSERT( m_registry );
		CPPUNIT_ASSERT( m_node );
	}

	void testCreateMetaKey()
	{
		Onion::MetaKey *mkey = m_node->createMetaKey( "metakey", "value" );
		CPPUNIT_ASSERT( mkey != 0 );
		CPPUNIT_ASSERT( mkey->getValue() == "value" );
	}

	void testGetMetaKey()
	{
		Onion::MetaKey *mkey = m_node->createMetaKey( "metakey",
			"value" );
		CPPUNIT_ASSERT( mkey != 0 );
		CPPUNIT_ASSERT( mkey == m_node->getMetaKey( "metakey" ) );
	}

	void testMetaKeyNotFound()
	{
		m_node->getMetaKey( "not existed" );
	}

	void testGetMetaKeyList()
	{
		Onion::MetaKey *mkey = m_node->createMetaKey( "metakey",
			"value" );
		std::list< std::string > lst = m_node->getMetaKeyList();
		CPPUNIT_ASSERT( *lst.begin() == "metakey" );
		CPPUNIT_ASSERT( ++lst.begin() == lst.end() );
	}

	void testRemoveMetaKey()
	{
		Onion::MetaKey *mkey = m_node->createMetaKey( "metakey",
			"value" );
		CPPUNIT_ASSERT( mkey != 0 );
		m_node->removeMetaKey( "metakey" );

		std::list< std::string > lst = m_node->getMetaKeyList();
		CPPUNIT_ASSERT( lst.begin() == lst.end() );

	}
	
	void testSave()
	{
		Onion::MetaKey *mkey;
		mkey = m_node->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_node->createMetaKey( "filename", "testNode.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_node->createMetaKey( "metakey", "saved value" );
		CPPUNIT_ASSERT( mkey != 0 );

		m_node->save( false );
	}

	void testLoad()
	{
		Onion::MetaKey *mkey;
		
		mkey = m_node->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_node->createMetaKey( "filename", "testNode.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_node->createMetaKey("metakey", "value" );
		CPPUNIT_ASSERT( mkey != 0 );
		CPPUNIT_ASSERT( mkey->getValue() == "value" );

		m_node->load( false );
		CPPUNIT_ASSERT( m_node->getMetaKey( "metakey" ) == mkey );
		CPPUNIT_ASSERT( mkey->getValue() == "saved value" );
	}

}; // class NodeTest

#endif // TESTS_NODETESTS_H

// <EOF>
