#ifndef TESTS_STORAGETEST_H
#define TESTS_STORAGETEST_H

#include "cppunit/extensions/HelperMacros.h"
#include "../src/registry.h"
#include <iostream>
#include <algorithm>

class StorageTest : public CppUnit::TestFixture
{
	CPPUNIT_TEST_SUITE( StorageTest );
	CPPUNIT_TEST( testExistance );
	CPPUNIT_TEST( testOpenWriteClose );
	CPPUNIT_TEST( testOpenReadClose );
	CPPUNIT_TEST( testSave );
	CPPUNIT_TEST( testLoad );
	CPPUNIT_TEST( testSaveMarker );
	CPPUNIT_TEST( testLoadMarker );
	CPPUNIT_TEST_SUITE_END();	

protected:
	Onion::Registry *m_registry;
	Onion::Catalog *m_root;
	Onion::Storage *m_storage;
public:
	int countTestCases() const
	{
		return 1;
	}
	
	void setUp()
	{
		m_registry = new Onion::Registry();
		m_root = m_registry->getRoot();
		m_root->createMetaKey( "storage", "SimpleStorage" );
		m_root->createMetaKey( "filename", "test.reg" );
		m_storage = Onion::Storage::makeStorage( *m_root );
	}
	
	void tearDown()
	{
		delete m_registry;
	}

	void testExistance()
	{
		CPPUNIT_ASSERT( m_registry != 0 );
		CPPUNIT_ASSERT( m_root != 0 );
		CPPUNIT_ASSERT( m_storage != 0 );
	}
	
	void testOpenWriteClose()
	{
		m_storage->open( false );
		m_storage->close();
	}

	void testOpenReadClose()
	{
		m_storage->open( true );
		m_storage->close();
	}
	
	void testSave()
	{
		m_storage->open( false );
		m_storage->save( "saved variable", "saved value" );
		m_storage->close();
	}
	
	void testLoad()
	{
		m_storage->open( true );
		std::string var, val;
		CPPUNIT_ASSERT(	m_storage->load( var, val) == true);
		CPPUNIT_ASSERT( var == "saved variable" );
		CPPUNIT_ASSERT( val == "saved value" );
		m_storage->close();
	}
	void testSaveMarker()
	{
		m_storage->open( false );
		m_storage->save( "var1", "val1" );
		m_storage->saveMarker( "marker1", "mval1" );
		m_storage->save( "var2", "val2" );
		m_storage->saveMarker( "marker2", "mval2" );
		m_storage->save( "var3", "val3" );
		m_storage->saveMarker( "marker3", "mval3" );
		m_storage->close();
	}
	void testLoadMarker()
	{
		m_storage->open( true );
		std::string var, val;
		CPPUNIT_ASSERT( m_storage->load( var, val ) == true);
		CPPUNIT_ASSERT( var == "var1" );
		CPPUNIT_ASSERT( val == "val1" );
		CPPUNIT_ASSERT( m_storage->load( var, val ) == false);
		CPPUNIT_ASSERT( var == "marker1" );
		CPPUNIT_ASSERT( val == "mval1" );

		CPPUNIT_ASSERT( m_storage->load( var, val ) == true);
		CPPUNIT_ASSERT( var == "var2" );
		CPPUNIT_ASSERT( val == "val2" );
		CPPUNIT_ASSERT( m_storage->load( var, val ) == false);
		CPPUNIT_ASSERT( var == "marker2" );
		CPPUNIT_ASSERT( val == "mval2" );

		CPPUNIT_ASSERT( m_storage->load( var, val ) == true);
		CPPUNIT_ASSERT( var == "var3" );
		CPPUNIT_ASSERT( val == "val3" );
		CPPUNIT_ASSERT( m_storage->load( var, val ) == false);
		CPPUNIT_ASSERT( var == "marker3" );
		CPPUNIT_ASSERT( val == "mval3" );
		m_storage->close();
	}
	

};

#endif // TESTS_STORAGETEST_H

// <EOF>
